"""
   Descp: This file is used to store the DAOhaus app's text.

   Created on: 1-oct-2020

   Copyright 2020-2021 Youssef 'FRYoussef' El Faqir El Rhazoui
        <f.r.youssef@hotmail.com>
"""

TEXT: dict = {
    'approved_proposals': 'Approved proposals',
    'css_pane_border': 'daohaus-border',
    'css_id_assets': 'daohaus-assets',
    'css_id_activity': 'daohaus-activity',
    'css_id_member': 'daohaus-members',
    'css_id_organization': 'daohaus-organization',
    'css_id_rage_quit': 'daohaus-rage-quit',
    'css_id_vote': 'daohaus-vote',
    'css_id_proposal': 'daohaus-proposal',
    'disclaimer_activity': 'As activity we consider new proposals and votes',
    'dp_id_active_members': 'daohaus-dp-active-members',
    'dp_id_members': 'daohaus-dp-members',
    'dp_id_proposals': 'daohaus-dp-proposals',
    'dp_id_treasury': 'daohaus-dp-treasury',
    'dp_id_vote': 'daohaus-dp-vote',
    'dp_title_active_members': 'Active members',
    'dp_title_members': 'Members',
    'dp_title_proposals': 'New Proposals',
    'dp_title_treasury': 'Treasury',
    'dp_title_vote': 'Votes',
    'pane_css_prefix': 'daohaus-pane',
    'proposal_donation': 'Donation proposals',
    'proposal_grant': 'Grant proposals',
    'proposal_new_member': 'New member proposals',
    'proposal_other': 'Others',
    'rejected_proposals': 'Rejected proposals',
    'title_active_members': 'Active members',
    'title_active_organization': 'Number of DAOs active per month',
    'title_active_voters': 'Active voters',
    'title_activity': 'Activity',
    'title_approval_proposal_rate': 'Approval proposal rate',
    'title_assets': 'Assets',
    'title_assets_value': 'Tokens with fiat equivalence (USD)',
    'title_assets_novalue': 'Tokens without fiat equivalence',
    'title_member': 'Members',
    'title_new_members': 'New members',
    'title_new_proposals': 'New proposals',
    'title_organization_activity': 'Total activity per month',
    'title_out_members': 'Outgoing members',
    'title_proposal': 'Proposals',
    'title_proposal_outcomes': 'Proposal outcomes',
    'title_proposal_type': 'Proposals by type',
    'title_rage_quits': 'Rage Quit',
    'title_total_members': 'Total members',
    'title_vote': 'Votes',
    'title_vote_for_rate': 'Votes-for rate',
    'title_vote_against_rate': 'Votes-against rate',
    'title_vote_type': 'Votes for and against',
    'title_voters_percentage': 'Percentage of members who vote',
    'title_votes_voters': 'Number of votes per voter',
    'votes_against': 'Votes against',
    'votes_for': 'Votes for',
}
